@@ -17,8 +17,6 @@ limitations under the License.
 from typing import Optional, Tuple, Union
 
 import torch
-import triton
-import triton.language as tl
 
 from .jit import FLASHINFER_CSRC_DIR, has_prebuilt_ops, load_cuda_ops
 from .utils import (
@@ -142,27 +140,6 @@ def _fake_append_paged_kv_cache_kernel(
     pass
 
 
-@triton.jit
-def get_batch_indices_positions_kernel(
-    append_indptr,
-    seq_lens_ptr,
-    batch_indices_ptr,
-    positions_ptr,
-    num_stages: tl.constexpr,
-):
-    batch_idx = tl.program_id(0)
-
-    batch_start = tl.load(append_indptr + batch_idx)
-    batch_end = tl.load(append_indptr + batch_idx + 1)
-    seq_len = tl.load(seq_lens_ptr + batch_idx)
-
-    for i in tl.range(batch_start, batch_end, 128, num_stages=num_stages):
-        offsets = tl.arange(0, 128) + i
-        mask = offsets < batch_end
-        tl.store(batch_indices_ptr + offsets, batch_idx, mask)
-        tl.store(positions_ptr + offsets, offsets + seq_len - batch_end, mask)
-
-
 def get_batch_indices_positions(
     append_indptr: torch.Tensor, seq_lens: torch.Tensor, nnz: int
 ) -> Tuple[torch.Tensor, torch.Tensor]:
@@ -210,6 +187,8 @@ def get_batch_indices_positions(
     batch_size = append_indptr.size(0) - 1
     batch_indices = torch.empty((nnz,), device=append_indptr.device, dtype=torch.int32)
     positions = torch.empty((nnz,), device=append_indptr.device, dtype=torch.int32)
+    from .triton.page import get_batch_indices_positions_kernel
+
     get_batch_indices_positions_kernel[(batch_size,)](
         append_indptr, seq_lens, batch_indices, positions, num_stages=2
     )
